@@ -79,6 +79,7 @@ Screen::Screen(int lines, int columns)
     , _bottomMargin(0)
     , _replMode(REPL_None)
     , _hasRepl(false)
+    , _replHadOutput(false)
     , _replLastOutputStart(std::pair(-1, -1))
     , _tabStops(QBitArray())
     , _selBegin(0)
@@ -1132,8 +1133,22 @@ void Screen::displayCharacter(uint c)
             }
         }
         if (c == 0xFE0F) {
-            // Emoji presentation - should not be included
             currentChar.flags |= EF_EMOJI_REPRESENTATION;
+            if (charToCombineWithX == _cuX - 1) {
+                // If width was 1, change to two.
+                if (_screenLines[_cuY].size() < _cuX + 1) {
+                    _screenLines[_cuY].resize(_cuX + 1);
+                }
+                Character &ch = _screenLines[_cuY][_cuX];
+                ch.setRightHalfOfDoubleWide();
+                ch.foregroundColor = _effectiveForeground;
+                ch.backgroundColor = _effectiveBackground;
+                ch.rendition = _effectiveRendition;
+                ch.flags = setRepl(EF_UNREAL, _replMode);
+                _cuX += 1;
+            }
+            // Emoji presentation should not be included
+            // (maybe a bug in Qt? including this code point in sequences breaks emoji-zwj-sequences.txt)
             return;
         }
         if (c == 0x200D) {
@@ -2361,12 +2376,24 @@ void Screen::setReplMode(int mode)
         } else if (_replMode == REPL_PROMPT) {
             _lineProperties[_cuY].counter = ++commandCounter;
         }
+        if (mode == REPL_PROMPT) {
+            if (_replHadOutput) {
+                _currentTerminalDisplay->sessionController()->notifyPrompt();
+                _replHadOutput = false;
+            }
+        }
+        if (mode == REPL_OUTPUT) {
+            _replHadOutput = true;
+        }
         _replMode = mode;
         _replModeStart = std::make_pair(_cuY, _cuX);
         _replModeEnd = std::make_pair(_cuY, _cuX);
     }
     if (mode != REPL_None) {
-        _hasRepl = true;
+        if (!_hasRepl) {
+            _hasRepl = true;
+            _currentTerminalDisplay->sessionController()->setVisible(QStringLiteral("monitor-prompt"), true);
+        }
         Q_EMIT _currentTerminalDisplay->screenWindow()->selectionChanged(); // Enable copy action
         setLineProperty(LINE_PROMPT_START << (mode - REPL_PROMPT), true);
     }
