@@ -11,7 +11,7 @@
 #include <QString>
 
 // KDE
-#include <qtest.h>
+#include <QTest>
 
 using namespace Konsole;
 
