@@ -527,6 +527,7 @@ QRegion TerminalPainter::highlightScrolledLinesRegion(TerminalScrollBar *scrollB
                               nb_lines * m_parentDisplay->terminalFont()->fontHeight());
         new_highlight.setTop(std::max(new_highlight.top(), m_parentDisplay->contentRect().top()));
         new_highlight.setBottom(std::min(new_highlight.bottom(), m_parentDisplay->contentRect().bottom()));
+        new_highlight = highdpi_adjust_rect(new_highlight);
         if (!new_highlight.isValid()) {
             new_highlight = QRect(0, 0, 0, 0);
         }
@@ -854,7 +855,6 @@ void TerminalPainter::drawAboveText(QPainter &painter,
                                     CharacterColor const *ulColorTable)
 {
     bool first = true;
-    QRect constRect(0, 0, 0, 0);
     QColor backgroundColor;
     QColor foregroundColor;
     int lastX = 0;
@@ -884,13 +884,11 @@ void TerminalPainter::drawAboveText(QPainter &painter,
                 }
                 if ((i == width || style[x].rendition.f.overline == 0) && startOverline >= 0) {
                     QPen pen(foregroundColor);
-                    int y = rect.y() + m_parentDisplay->terminalFont()->fontAscent();
+                    qreal y = rect.y() + m_parentDisplay->terminalFont()->fontAscent() - m_parentDisplay->terminalFont()->overlinePos()
+                        + m_parentDisplay->terminalFont()->lineSpacing() / static_cast<qreal>(2);
                     pen.setWidth(m_parentDisplay->terminalFont()->lineWidth());
                     painter.setPen(pen);
-                    painter.drawLine(rect.x() + fontWidth * startOverline,
-                                     y - m_parentDisplay->terminalFont()->overlinePos(),
-                                     rect.x() + fontWidth * i - 1,
-                                     y - m_parentDisplay->terminalFont()->overlinePos());
+                    painter.drawLine(QLineF(rect.x() + fontWidth * startOverline, y, rect.x() + fontWidth * i - 1, y));
                     startOverline = -1;
                 }
                 int underline = style[lastX].rendition.f.underline;
@@ -900,16 +898,12 @@ void TerminalPainter::drawAboveText(QPainter &painter,
                     if (ulColorTable != nullptr && (style[lastX].flags & EF_UNDERLINE_COLOR) != 0) {
                         pen.setColor(ulColorTable[((style[lastX].flags & EF_UNDERLINE_COLOR)) / EF_UNDERLINE_COLOR_1 - 1].color(colorTable));
                     }
-                    int y = rect.y() + m_parentDisplay->terminalFont()->fontAscent() + m_parentDisplay->terminalFont()->underlinePos()
-                        + m_parentDisplay->terminalFont()->lineWidth();
                     int lw = m_parentDisplay->terminalFont()->lineWidth();
+                    qreal y = rect.y() + m_parentDisplay->terminalFont()->fontAscent() + m_parentDisplay->terminalFont()->underlinePos()
+                        + m_parentDisplay->terminalFont()->lineSpacing() / static_cast<qreal>(2);
                     if (underline == RE_UNDERLINE_DOUBLE || underline == RE_UNDERLINE_CURL) {
                         y = rect.bottom() - 1;
                         lw = 1;
-                    } else {
-                        if (lw + lw + m_parentDisplay->terminalFont()->fontAscent() + m_parentDisplay->terminalFont()->underlinePos() > rect.height()) {
-                            lw = rect.height() - m_parentDisplay->terminalFont()->fontAscent() - m_parentDisplay->terminalFont()->underlinePos() - lw;
-                        }
                     }
                     pen.setWidth(lw);
                     if (underline == RE_UNDERLINE_DOT) {
@@ -922,12 +916,12 @@ void TerminalPainter::drawAboveText(QPainter &painter,
                         pen.setDashPattern(dashes);
                     }
                     painter.setPen(pen);
-                    painter.drawLine(rect.x() + fontWidth * startUnderline, y, rect.x() + fontWidth * i - 1, y);
+                    painter.drawLine(QLineF(rect.x() + fontWidth * startUnderline, y, rect.x() + fontWidth * i - 1, y));
                     if (underline == RE_UNDERLINE_DOUBLE) {
                         painter.drawLine(rect.x() + fontWidth * startUnderline, y - 2, rect.x() + fontWidth * i - 1, y - 2);
                     }
                     if (underline == RE_UNDERLINE_CURL) {
-                        painter.drawLine(rect.x() + fontWidth * startUnderline + 2, y - 1, rect.x() + fontWidth * i - 1, y - 1);
+                        painter.drawLine(QLineF(rect.x() + fontWidth * startUnderline + 2, y - 1, rect.x() + fontWidth * i - 1, y - 1));
                     }
 
                     startUnderline = -1;
